@@ -5,6 +5,7 @@ import uuid
 import operator
 import pydub
 import logging
+import pprint
 
 import cloudlanguagetools.service
 import cloudlanguagetools.constants
@@ -13,12 +14,15 @@ import cloudlanguagetools.languages
 import cloudlanguagetools.ttsvoice
 import cloudlanguagetools.translationlanguage
 import cloudlanguagetools.transliterationlanguage
+import cloudlanguagetools.dictionarylookup
 import cloudlanguagetools.errors
 
 
 import azure.cognitiveservices.speech
 import azure.cognitiveservices.speech.audio
 
+logger = logging.getLogger(__name__)
+
 AUDIO_LOCALE_OVERRIDE_MAP = {
     'zh-CN-liaoning': 'zh_CN',
     'zh-CN-sichuan': 'zh_CN',
@@ -138,6 +142,28 @@ class AzureTransliterationLanguage(cloudlanguagetools.transliterationlanguage.Tr
             'to_script': self.to_script
         }
 
+class AzureDictionaryLookup(cloudlanguagetools.dictionarylookup.DictionaryLookup):
+    def __init__(self, source_language_code, target_language_code, lookup_type):
+        self.service = cloudlanguagetools.constants.Service.Azure
+        self.language = get_translation_language_enum(source_language_code)
+        self.target_language = get_translation_language_enum(target_language_code)
+        self.source_language_code = source_language_code
+        self.target_language_code = target_language_code
+        self.lookup_type = lookup_type
+
+    def get_lookup_key(self):
+        return {
+            'source_language_code': self.source_language_code,
+            'target_language_code': self.target_language_code,
+            'language': self.language.name,
+            'lookup_type': self.lookup_type.name
+        }        
+
+    def get_lookup_name(self):
+        return f'{self.service.name} ({self.language.lang_name} to {self.target_language.lang_name}), {self.lookup_type.name}'
+
+    def get_lookup_shortname(self):
+        return f'{self.service.name}, {self.language.lang_name}, {self.lookup_type.name}'
 
 class AzureService(cloudlanguagetools.service.Service):
     def __init__(self):
@@ -382,7 +408,84 @@ class AzureService(cloudlanguagetools.service.Service):
 
         raise "unknown error"
 
-    def dictionary_lookup(self, input_text, from_language_key, to_language_key):
+    def get_dictionary_lookup_list(self):
+        result = []
+
+        azure_data = self.get_supported_languages()
+        for source_language_id, data in azure_data['dictionary'].items():
+            for translation in data['translations']:
+                target_language_id = translation['code']
+                result.append(AzureDictionaryLookup(source_language_id, 
+                                target_language_id, 
+                                cloudlanguagetools.constants.DictionaryLookupType.Definitions))
+                result.append(AzureDictionaryLookup(source_language_id, 
+                                target_language_id, 
+                                cloudlanguagetools.constants.DictionaryLookupType.PartOfSpeech))
+                result.append(AzureDictionaryLookup(source_language_id, 
+                                target_language_id, 
+                                cloudlanguagetools.constants.DictionaryLookupType.PartOfSpeechDefinitions))
+
+        return result
+
+    def iterate_dictionary_results(self, text, lookup_key):
+        from_language_code = lookup_key['source_language_code']
+        to_language_code = lookup_key['target_language_code']
+        base_url = f'{self.url_translator_base}/dictionary/lookup?api-version=3.0'
+        params = f'&to={to_language_code}&from={from_language_code}'
+        url = base_url + params
+
+        logger.info(f'querying url for dictionary lookup: {url}')
+
+        # You can pass more than one object in body.
+        body = [{
+            'text': text
+        }]
+        request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
+        response = request.json()
+        if len(response) > 1:
+            raise Exception(f'more than one response entries, {url}, {text}')
+
+        translation_entries = response[0]['translations']
+        for translation_entry in translation_entries:
+            yield translation_entry
+
+    def collect_definitions(self, generator):
+        result = []
+        for entry in generator:
+            result.append(entry['displayTarget'])
+        return result
+
+    def collect_partofspeech(self, generator):
+        result = []
+        for entry in generator:
+            result.append(entry['posTag'].lower())
+        result = list(set(result))
+        result.sort()
+        return result
+
+    def collect_partofspeech_definitions(self, generator):
+        result = {}
+        for entry in generator:
+            part_of_speech = entry['posTag'].lower()
+            if part_of_speech not in result:
+                result[part_of_speech] = []
+            result[part_of_speech].append(entry['displayTarget'])
+        return result        
+
+    def get_dictionary_lookup(self, text, lookup_key):
+        lookup_type = cloudlanguagetools.constants.DictionaryLookupType[lookup_key['lookup_type']]
+        lookup_type_fn_map = {
+            cloudlanguagetools.constants.DictionaryLookupType.Definitions: self.collect_definitions,
+            cloudlanguagetools.constants.DictionaryLookupType.PartOfSpeech: self.collect_partofspeech,
+            cloudlanguagetools.constants.DictionaryLookupType.PartOfSpeechDefinitions: self.collect_partofspeech_definitions,
+        }
+        lookup_fn = lookup_type_fn_map[lookup_type]
+        generator = self.iterate_dictionary_results(text, lookup_key)
+
+        return lookup_fn(generator)
+
+
+    def custom_dictionary_lookup(self, input_text, from_language_key, to_language_key):
         base_url = f'{self.url_translator_base}/dictionary/lookup?api-version=3.0'
         params = f'&to={to_language_key}&from={from_language_key}'
         url = base_url + params
@@ -396,6 +499,8 @@ class AzureService(cloudlanguagetools.service.Service):
         request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
         response = request.json()
 
+        pprint.pprint(response)
+
         translation_entries = response[0]['translations']
         translations = [entry['displayTarget'] for entry in translation_entries]
 
