@@ -23,6 +23,7 @@ import cloudlanguagetools.voicen
 import cloudlanguagetools.pythainlp
 import cloudlanguagetools.spacy
 import cloudlanguagetools.argostranslate
+import cloudlanguagetools.wenlin
 import cloudlanguagetools.encryption
 
 class ServiceManager():
@@ -45,6 +46,7 @@ class ServiceManager():
         self.services[cloudlanguagetools.constants.Service.Spacy.name] = cloudlanguagetools.spacy.SpacyService()
         self.services[cloudlanguagetools.constants.Service.MandarinCantonese.name] = cloudlanguagetools.mandarincantonese.MandarinCantoneseService()            
         self.services[cloudlanguagetools.constants.Service.ArgosTranslate.name] = cloudlanguagetools.argostranslate.ArgosTranslateService()
+        self.services[cloudlanguagetools.constants.Service.Wenlin.name] = cloudlanguagetools.wenlin.WenlinService()
 
     def configure_default(self):
         # use the stored keys to configure services
@@ -68,6 +70,8 @@ class ServiceManager():
         tts_voice_list = self.get_tts_voice_list()
         logging.info('retrieving tokenization options')
         tokenization_options = self.get_tokenization_options()
+        logging.info('retrieving dictionary lookup options')
+        dictionary_lookup_options = self.get_dictionary_lookup_options()        
 
         return {
             'language_list': language_list,
@@ -75,6 +79,7 @@ class ServiceManager():
             'transliteration_options': [option.json_obj() for option in transliteration_language_list],
             'voice_list': [voice.json_obj() for voice in tts_voice_list],
             'tokenization_options': [option.json_obj() for option in tokenization_options],
+            'dictionary_lookup_options': [option.json_obj() for option in dictionary_lookup_options],
         }
         
 
@@ -128,6 +133,18 @@ class ServiceManager():
         tokenization_list = self.get_tokenization_options()
         return [tokenization_option.json_obj() for tokenization_option in tokenization_list]
 
+    # dictionary lookups
+
+    def get_dictionary_lookup_options(self):
+        result = []
+        for key, service in self.services.items():
+            result.extend(service.get_dictionary_lookup_list())
+        return result
+
+    def get_dictionary_lookup_options_json(self):
+        dictionary_lookup_list = self.get_dictionary_lookup_options()
+        return [dict_lookup_option.json_obj() for dict_lookup_option in dictionary_lookup_list]
+
     def get_tts_audio(self, text, service, voice_id, options):
         service = self.services[service]
         return service.get_tts_audio(text, voice_id, options)
@@ -172,6 +189,10 @@ class ServiceManager():
         service = self.services[service]
         return service.get_tokenization(text, tokenization_key)
 
+    def get_dictionary_lookup(self, text, service, lookup_key):
+        service = self.services[service]
+        return service.get_dictionary_lookup(text, lookup_key)
+
     def get_breakdown(self, text, tokenization_option, translation_option, transliteration_option):
         
         # first, tokenize
